#!/bin/bash
versiondir=9.6
logmgr_log=~/$versiondir/purgelog/logmgr.log.`date +%Y-%m-%d`
PURGELOG=~/$versiondir/purgelog/
tmp_purge_log=~/$versiondir/purgelog/temp_purge.log
echo "Starting of postgreSQL purging : "`date`>>$logmgr_log
if [ $# -ne 7 ]; then
        echo -e "\033[31mPlease provide all config parameters 'dbname,username,password,logtokeep and logtodelete'.\033[0m" >>$logmgr_log
        exit 0;
fi

dbname=`echo $1`
username=`echo $2`
password=`echo $3`
logtokeep=`echo $4`
logtodelete=`echo $5`
psqlosuser=`echo $6`
sla=`echo $7`

if [ ! $(echo $logtokeep| grep "^[ [:digit:] ]*$" ) ] || [ ! $(echo $logtodelete| grep "^[ [:digit:] ]*$" ) ] || [ $logtokeep -le 0 ] || [ $logtodelete -le 0 ]; then
        echo -e "\033[31mlogtokeep and logtodelete must be numbers and should be great than zero.\033[0m" >>$logmgr_log
        exit 0;
fi
if [ $logtodelete -le $logtokeep ]; then
        echo -e "\033[31mvalue of logtodelete must be greater than value of logtokeep .\033[0m" >>$logmgr_log
        exit 0;
fi

. ~/$versiondir/pg_env.sh
if [ $? -ne 0 ]; then
        echo -e "\033[31mUnable to set postgres environment. Please contact system administrator.\033[0m" >>$logmgr_log
        exit 0;
fi


source ~/$versiondir/pg_env.sh
PGPASSWORD=`echo $password`
export PGPASSWORD

#########Events log purging################
echo -e "\n\n############Events log purging##################" >>$logmgr_log
psql -e -U $username $dbname -t -c "SELECT COUNT(*) AS cnt FROM log_events JOIN log_calls ON log_calls.callseq = log_events.callseq WHERE (log_calls.startdate > current_timestamp + interval '-$logtodelete' day ) AND (log_calls.startdate < current_timestamp + interval '-$logtokeep' day )" |sed '/^$/d' >$tmp_purge_log  2>&1
status_=`echo $?`
if [  $status_ -ne 0 ]; then
        echo -e "\033[31mUnable to get counts of log events counts. Here is the exception. Please contact system administrator.\033[0m" >>$logmgr_log
        cat $tmp_purge_log >> $logmgr_log
        exit 0;
fi
events_counts_query=`head -1 $tmp_purge_log`
events_count=`tail -1 $tmp_purge_log`
echo -e "Events counts query\n======\n$events_counts_query" >>$logmgr_log
echo -e "Number of events logs to be deleted: $events_count" >>$logmgr_log
if [ $events_count -gt 0 ]; then
        events_purge_start_time=`psql  -U $username $dbname -t -c "select current_timestamp"` >>$logmgr_log
        psql -e -U $username $dbname -t -c "DELETE FROM log_events WHERE  EXISTS (SELECT 1 FROM log_calls WHERE log_calls.callseq = log_events.callseq AND (log_calls.startdate > current_timestamp + interval '-$logtodelete' day ) AND (log_calls.startdate < current_timestamp + interval '-$logtokeep' day ))" >$tmp_purge_log  2>&1
        status_=`echo $?`
        events_purge_end_time=`psql  -U $username $dbname -t -c "select current_timestamp"` >>$logmgr_log
        if [ $status_ -ne 0 ]; then
                echo -e "\033[31mUnable to delete log events data. Here is the exception. Please contact system administrator.\033[0m" >>$logmgr_log
                cat $tmp_purge_log >>$logmgr_log
                exit 0;
        fi
        events_delete_query=`head -1 $tmp_purge_log`
        events_deleted=`tail -1 $tmp_purge_log| awk '{print $2}'`
        echo -e "Events log purging started: $events_purge_start_time" >>$logmgr_log
        echo -e "Events delete query\n======\n$events_delete_query" >>$logmgr_log
        echo -e "Number of events deleted: $events_deleted" >>$logmgr_log
        echo -e "Events log purging ended: $events_purge_end_time" >>$logmgr_log
else
        echo -e "No event logs to be deleted"  >>$logmgr_log
fi


#############Documents log purging##############
echo -e "\n\n############Documents logs purging##################" >>$logmgr_log
psql -e -U $username $dbname -t -c "SELECT COUNT(*) AS cnt FROM log_documents WHERE (log_documents.updatedate  > current_timestamp + interval '-$logtodelete' day ) AND (log_documents.updatedate < current_timestamp + interval '-$logtokeep' day )" |sed '/^$/d' >$tmp_purge_log  2>&1
status_=`echo $?`
if [  $status_ -ne 0 ]; then
        echo -e "\033[31mUnable to get counts of document logs counts. Here is the exception. Please contact system administrator.\033[0m" >>$logmgr_log
        cat $tmp_purge_log >>$logmgr_log
        exit 0;
fi
docs_counts_query=`head -1 $tmp_purge_log`
docs_count=`tail -1 $tmp_purge_log`
echo -e "Docs counts query\n======\n$docs_counts_query" >>$logmgr_log
echo -e "Number of doc logs to be deleted: $docs_count" >>$logmgr_log
if [ $docs_count -gt 0 ]; then
        docs_purge_start_time=`psql  -U $username $dbname -t -c "select current_timestamp"`
        psql -e -U $username $dbname -t -c "DELETE FROM log_documents WHERE (log_documents.updatedate >  current_timestamp + interval '-$logtodelete' day ) AND (log_documents.updatedate < current_timestamp + interval '-$logtokeep' day )" >$tmp_purge_log  2>&1
        status_=`echo $?`
        docs_purge_end_time=`psql  -U $username $dbname -t -c "select current_timestamp"`
        if [ $status_ -ne 0 ]; then
                echo -e "\033[31mUnable to delete document logs data. Here is the exception. Please contact system administrator.\033[0m" >>$logmgr_log
                cat $tmp_purge_log >>$logmgr_log
                exit 0;
        fi
        docs_delete_query=`head -1 $tmp_purge_log`
        docs_deleted=`tail -1 $tmp_purge_log| awk '{print $2}'`
        echo -e "Documents log purging started: $docs_purge_start_time" >>$logmgr_log
        echo -e "Docs delete query\n======\n$docs_delete_query" >>$logmgr_log
        echo -e "Number of document logs deleted: $docs_deleted" >>$logmgr_log
        echo -e "Docs log purging ended: $docs_purge_end_time" >>$logmgr_log
else
        echo -e "No document logs to be deleted"  >>$logmgr_log
fi

#############Call log purging##############

echo -e "\n\n############Call logs purging##################" >>$logmgr_log
psql -e -U $username $dbname -t -c "SELECT COUNT(*) AS cnt FROM log_calls WHERE (log_calls.startdate > current_timestamp + interval '-$logtodelete' day ) AND (log_calls.startdate < current_timestamp + interval '-$logtokeep' day )"|sed '/^$/d' >$tmp_purge_log  2>&1
status_=`echo $?`
if [ $status_ -ne 0 ]; then
        echo -e "\033[31mUnable to get counts of call events counts. Here is the exception. Please contact system administrator.\033[0m" >>$logmgr_log
        cat $tmp_purge_log >>$logmgr_log
        exit 0;
fi

call_counts_query=`head -1 $tmp_purge_log`
call_count=`tail -1 $tmp_purge_log`
echo -e "Call counts query\n======\n$call_counts_query" >>$logmgr_log
echo -e "Number of call logs to be deleted: $call_count" >>$logmgr_log
if [ $call_count -gt 0 ]; then
        calls_purge_start_time=`psql  -U $username $dbname -t -c "select current_timestamp"`
        psql -e -U $username $dbname -t -c "DELETE FROM log_calls WHERE (log_calls.startdate > current_timestamp + interval '-$logtodelete' day ) AND (log_calls.startdate < current_timestamp + interval '-$logtokeep' day )" >$tmp_purge_log  2>&1
        status_=`echo $?`
        calls_purge_end_time=`psql  -U $username $dbname -t -c "select current_timestamp"`
        if [ $status_ -ne 0 ]; then
                echo -e "\033[31mUnable to delete log events data. Here is the exception. Please contact system administrator.\033[0m" >>$logmgr_log
                cat $tmp_purge_log >>$logmgr_log
                exit 0;
        fi
        calllog_delete_query=`head -1 $tmp_purge_log`
        calllogs_deleted=`tail -1 $tmp_purge_log| awk '{print $2}'`
        echo -e "Call logs purging started: $calls_purge_start_time" >>$logmgr_log
        echo -e "Call log delete query\n======\n$calllog_delete_query" >>$logmgr_log
        echo -e "Number of call logs deleted: $calllogs_deleted" >>$logmgr_log
        echo -e "Call log purging ended: $calls_purge_end_time" >>$logmgr_log
else
        echo -e "No call logs to be deleted."  >>$logmgr_log
fi

for vacuumInput in `echo $events_count:log_events $docs_count:log_documents $call_count:log_calls`
do
        deletedRows=`echo $vacuumInput|cut -d: -f1`
        if [ $deletedRows -gt 0 ]; then
                vacuumTable=`echo $vacuumInput | cut -d: -f2`
                echo -e "\n\n####Performing VACUUM operation on $vacuumTable to reclaim the storage through $deletedRows obsolete rows####\nTimeStamp: `date`\n" >> $logmgr_log
                psql -e -U $username $dbname -t -c "VACUUM FULL VERBOSE ANALYZE $vacuumTable" >> $logmgr_log  2>&1
                vacuumStatus=`echo $?`
                echo -e "TimeStamp: `date`" >> $logmgr_log
                if [ $vacuumStatus -ne 0 ]; then
                        echo -e "\033[31m\nUnable to run VACUUM operation on $vacuumTable. Please contact system administrator.\033[0m" >>$logmgr_log
                        exit 0;
                else
                        echo -e "\nVACUUM operation on $vacuumTable is completed." >> $logmgr_log
                fi
        fi
done


echo -e "\n\nRemoving $sla days old logmgr_log.log files from $PURGELOG\n===================">>$logmgr_log
n=`find $PURGELOG -maxdepth 1 -mtime +$sla -name "logmgr.log*"|wc -l`
if [ $n -eq 0 ]; then
        echo -e "There are no $sla days old logmgr.log files to be deleted.\n\n" >>$logmgr_log
else
        echo -e "Here is the list of logmgr.log files to be deleted." >>$logmgr_log
        find $PURGELOG -maxdepth 1 -mtime +$sla -name "logmgr.log*">>$logmgr_log
        find $PURGELOG -maxdepth 1 -mtime +$sla -name "logmgr.log*" -exec rm -rf {} \; &>>$logmgr_log
        if [ $? -eq 0 ]; then
                echo -e "Deleted $sla days old logmgr.log files successfully.\n\n===================">>$logmgr_log
        else
                echo -e "Unable to delete $sla days logmgr.log file. Please verify above error message...\n\n===================">>$logmgr_log
        fi
fi
echo "End of postgreSQL purging : "`date`>>$logmgr_log
